@@ -208,7 +208,7 @@
 	       ;; again, with properties
 	       (with-syntax ((struct-type-defs
 			      (stepper-syntax-property
-			       (syntax/loc x struct-type-defs) 'stepper-define-struct-hint #'?stx))
+			       (syntax/loc x struct-type-defs) 'stepper-black-box-expr #'?stx))
 			     (constructor-def
 			      (stepper-syntax-property #'constructor-def 'stepper-skip-completely #t))
 			     (predicate-def
