@@ -63,7 +63,7 @@ inline static void clean_up_thread_list(NewGC *gc)
 
   while(work) {
     if(!pagemap_find_page(gc->page_maps, work->thread) || marked(gc, work->thread)) {
-      work->thread = GC_resolve(work->thread);
+      work->thread = GC_resolve2(work->thread, gc);
       prev = work;
       work = work->next;
     } else {
@@ -202,7 +202,7 @@ inline static void clean_up_owner_table(NewGC *gc)
       if(!marked(gc, owner_table[i]->originator)) {
         owner_table[i]->originator = NULL;
       } else 
-        owner_table[i]->originator = GC_resolve(owner_table[i]->originator);
+        owner_table[i]->originator = GC_resolve2(owner_table[i]->originator, gc);
 
       /* potential delete */
       if(i != 1) 
@@ -218,11 +218,13 @@ inline static uintptr_t custodian_usage(NewGC*gc, void *custodian)
   int i;
 
   if(!gc->really_doing_accounting) {
-    gc->park[0] = custodian;
-    gc->really_doing_accounting = 1;
-    garbage_collect(gc, 1, 0);
-    custodian = gc->park[0]; 
-    gc->park[0] = NULL;
+    if (!gc->dumping_avoid_collection) {
+      gc->park[0] = custodian;
+      gc->really_doing_accounting = 1;
+      garbage_collect(gc, 1, 0, NULL);
+      custodian = gc->park[0]; 
+      gc->park[0] = NULL;
+    }
   }
 
   i = custodian_to_owner_set(gc, (Scheme_Custodian *)custodian);
@@ -433,12 +435,14 @@ inline static void BTC_add_account_hook(int type,void *c1,void *c2,uintptr_t b)
   AccountHook *work;
 
   if(!gc->really_doing_accounting) {
-    gc->park[0] = c1; 
-    gc->park[1] = c2;
-    gc->really_doing_accounting = 1;
-    garbage_collect(gc, 1, 0);
-    c1 = gc->park[0]; gc->park[0] = NULL;
-    c2 = gc->park[1]; gc->park[1] = NULL;
+    if (!gc->dumping_avoid_collection) {
+      gc->park[0] = c1; 
+      gc->park[1] = c2;
+      gc->really_doing_accounting = 1;
+      garbage_collect(gc, 1, 0, NULL);
+      c1 = gc->park[0]; gc->park[0] = NULL;
+      c2 = gc->park[1]; gc->park[1] = NULL;
+    }
   }
 
   if (type == MZACCT_LIMIT)
@@ -477,8 +481,8 @@ inline static void clean_up_account_hooks(NewGC *gc)
 
   while(work) {
     if((!work->c1 || marked(gc, work->c1)) && marked(gc, work->c2)) {
-      work->c1 = GC_resolve(work->c1);
-      work->c2 = GC_resolve(work->c2);
+      work->c1 = GC_resolve2(work->c1, gc);
+      work->c2 = GC_resolve2(work->c2, gc);
       prev = work;
       work = work->next;
     } else {
@@ -618,7 +622,11 @@ int BTC_single_allocation_limit(NewGC *gc, size_t sizeb) {
    * is much smaller than the actual available memory, and as long as
    * GC_out_of_memory protects any user-requested allocation whose size
    * is independent of any existing object, then we can enforce the limit. */
-  return (custodian_single_time_limit(gc, thread_get_owner(scheme_current_thread)) < sizeb);
+  Scheme_Thread *p = scheme_current_thread;
+  if (p)
+    return (custodian_single_time_limit(gc, thread_get_owner(p)) < sizeb);
+  else
+    return 0;
 }
 
 static inline void BTC_clean_up(NewGC *gc) {
