@@ -105,9 +105,8 @@ Scheme_Extension_Table *scheme_extension_table;
 #endif
 
 #ifndef NO_DYNAMIC_LOAD
-/* FIXME disallow extensions under places */
-FIXME_LATER static Scheme_Hash_Table *loaded_extensions; /* hash on scheme_initialize pointer */
-FIXME_LATER static Scheme_Hash_Table *fullpath_loaded_extensions; /* hash on full path name */
+THREAD_LOCAL_DECL(static Scheme_Hash_Table *loaded_extensions;) /* hash on scheme_initialize pointer */
+THREAD_LOCAL_DECL(static Scheme_Hash_Table *fullpath_loaded_extensions;) /* hash on full path name */
 #endif
 
 #ifdef MZ_PRECISE_GC 
@@ -122,13 +121,6 @@ FIXME_LATER static Scheme_Hash_Table *fullpath_loaded_extensions; /* hash on ful
 void scheme_init_dynamic_extension(Scheme_Env *env)
 {
   if (scheme_starting_up) {
-#ifndef NO_DYNAMIC_LOAD
-    REGISTER_SO(loaded_extensions);
-    REGISTER_SO(fullpath_loaded_extensions);
-    loaded_extensions = scheme_make_hash_table(SCHEME_hash_ptr);
-    fullpath_loaded_extensions = scheme_make_hash_table(SCHEME_hash_string);
-#endif
-
 #ifdef LINK_EXTENSIONS_BY_TABLE
     REGISTER_SO(scheme_extension_table);
     
@@ -191,6 +183,15 @@ static Scheme_Object *do_load_extension(const char *filename,
   void *handle;
   int comppath;
 
+#ifndef NO_DYNAMIC_LOAD
+  if (!loaded_extensions) {
+    REGISTER_SO(loaded_extensions);
+    REGISTER_SO(fullpath_loaded_extensions);
+    loaded_extensions = scheme_make_hash_table(SCHEME_hash_ptr);
+    fullpath_loaded_extensions = scheme_make_hash_table(SCHEME_hash_string);
+  }
+#endif
+
   comppath = scheme_is_complete_path(filename, strlen(filename), SCHEME_PLATFORM_PATH_KIND);
 
   reload_f = NULL;
@@ -216,19 +217,9 @@ static Scheme_Object *do_load_extension(const char *filename,
     if (filename[0] != '/') {
       int l = strlen(filename);
       char *s;
-# if defined(MZ_USE_PLACES) && defined(MZ_PRECISE_GC)
-      void *original_gc;
-      original_gc = GC_switch_to_master_gc();
-      scheme_start_atomic();
-# endif
 
       s = (char *)scheme_malloc_atomic(l + 3);
 
-# if defined(MZ_USE_PLACES) && defined(MZ_PRECISE_GC)
-      scheme_end_atomic_no_swap();
-      GC_switch_back_from_master(original_gc);
-# endif
-
       s[0] = '.';
       s[1] = '/';
       memcpy(s + 2, filename, l + 1);
@@ -408,20 +399,8 @@ static Scheme_Object *do_load_extension(const char *filename,
     return NULL;
 #else
 
-    if (comppath) {
-# if defined(MZ_USE_PLACES) && defined(MZ_PRECISE_GC)
-    void *original_gc;
-    original_gc = GC_switch_to_master_gc();
-    scheme_start_atomic();
-# endif
-
+    if (comppath)
       scheme_hash_set(fullpath_loaded_extensions, (Scheme_Object *)filename, mzPROC_TO_HASH_OBJ(init_f));
-
-# if defined(MZ_USE_PLACES) && defined(MZ_PRECISE_GC)
-    scheme_end_atomic_no_swap();
-    GC_switch_back_from_master(original_gc);
-# endif
-    }
   }
 #endif
 
@@ -432,24 +411,12 @@ static Scheme_Object *do_load_extension(const char *filename,
     init_f = ed->reload_f;
     modname_f = ed->modname_f;
   } else {
-# if defined(MZ_USE_PLACES) && defined(MZ_PRECISE_GC)
-    void *original_gc;
-    original_gc = GC_switch_to_master_gc();
-    scheme_start_atomic();
-# endif
-
     ed = MALLOC_ONE_ATOMIC(ExtensionData);
     ed->handle = handle;
     ed->init_f = init_f;
     ed->reload_f = reload_f;
     ed->modname_f = modname_f;
     scheme_hash_set(loaded_extensions, mzPROC_TO_HASH_OBJ(init_f), (Scheme_Object *)ed);
-
-# if defined(MZ_USE_PLACES) && defined(MZ_PRECISE_GC)
-    scheme_end_atomic_no_swap();
-    GC_switch_back_from_master(original_gc);
-# endif
-
   }
 
   if (SCHEME_SYMBOLP(expected_module)) {
