@@ -1,4 +1,4 @@
- Scheme_Object* scheme_rtcall_siS_s(const char *who, int src_type, prim_siS_s f, Scheme_Object* g51, int g52, Scheme_Object** g53)
+ Scheme_Object* scheme_rtcall_siS_s(const char *who, int src_type, prim_siS_s f, Scheme_Object* g56, int g57, Scheme_Object** g58)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -13,9 +13,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g51;
-    future->arg_i1 = g52;
-    future->arg_S2 = g53;
+      future->arg_s0 = g56;
+    future->arg_i1 = g57;
+    future->arg_S2 = g58;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -24,7 +24,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_iSs_s(const char *who, int src_type, prim_iSs_s f, int g54, Scheme_Object** g55, Scheme_Object* g56)
+ Scheme_Object* scheme_rtcall_iSs_s(const char *who, int src_type, prim_iSs_s f, int g59, Scheme_Object** g60, Scheme_Object* g61)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -39,9 +39,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g54;
-    future->arg_S1 = g55;
-    future->arg_s2 = g56;
+      future->arg_i0 = g59;
+    future->arg_S1 = g60;
+    future->arg_s2 = g61;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -50,7 +50,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_s_s(const char *who, int src_type, prim_s_s f, Scheme_Object* g57)
+ Scheme_Object* scheme_rtcall_s_s(const char *who, int src_type, prim_s_s f, Scheme_Object* g62)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -65,8 +65,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g57;
-  send_special_result(future, g57);
+      future->arg_s0 = g62;
+  send_special_result(future, g62);
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
   retval = future->retval_s;
@@ -74,7 +74,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_n_s(const char *who, int src_type, prim_n_s f, Scheme_Native_Closure_Data* g58)
+ Scheme_Object* scheme_rtcall_n_s(const char *who, int src_type, prim_n_s f, Scheme_Native_Closure_Data* g63)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -89,7 +89,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_n0 = g58;
+      future->arg_n0 = g63;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -122,7 +122,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_ss_s(const char *who, int src_type, prim_ss_s f, Scheme_Object* g59, Scheme_Object* g60)
+ Scheme_Object* scheme_rtcall_ss_s(const char *who, int src_type, prim_ss_s f, Scheme_Object* g64, Scheme_Object* g65)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -137,8 +137,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g59;
-    future->arg_s1 = g60;
+      future->arg_s0 = g64;
+    future->arg_s1 = g65;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -147,7 +147,33 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_tt_s(const char *who, int src_type, prim_tt_s f, const Scheme_Object* g61, const Scheme_Object* g62)
+ Scheme_Object* scheme_rtcall_ssi_s(const char *who, int src_type, prim_ssi_s f, Scheme_Object* g66, Scheme_Object* g67, int g68)
+   XFORM_SKIP_PROC
+{
+  Scheme_Future_Thread_State *fts = scheme_future_thread_state;
+  future_t *future;
+  double tm;
+  Scheme_Object* retval;
+
+  future = fts->thread->current_ft;
+  future->prim_protocol = SIG_ssi_s;
+  future->prim_func = f;
+  tm = get_future_timestamp();
+  future->time_of_request = tm;
+  future->source_of_request = who;
+  future->source_type = src_type;
+      future->arg_s0 = g66;
+    future->arg_s1 = g67;
+    future->arg_i2 = g68;
+  
+  future_do_runtimecall(fts, (void*)f, 0, 1);
+  future = fts->thread->current_ft;
+  retval = future->retval_s;
+  future->retval_s = 0;
+  receive_special_result(future, retval, 1);
+  return retval;
+}
+ Scheme_Object* scheme_rtcall_tt_s(const char *who, int src_type, prim_tt_s f, const Scheme_Object* g69, const Scheme_Object* g70)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -162,8 +188,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_t0 = g61;
-    future->arg_t1 = g62;
+      future->arg_t0 = g69;
+    future->arg_t1 = g70;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -172,7 +198,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- MZ_MARK_STACK_TYPE scheme_rtcall_ss_m(const char *who, int src_type, prim_ss_m f, Scheme_Object* g63, Scheme_Object* g64)
+ MZ_MARK_STACK_TYPE scheme_rtcall_ss_m(const char *who, int src_type, prim_ss_m f, Scheme_Object* g71, Scheme_Object* g72)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -187,8 +213,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g63;
-    future->arg_s1 = g64;
+      future->arg_s0 = g71;
+    future->arg_s1 = g72;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -197,7 +223,7 @@
   
   return retval;
 }
- Scheme_Object* scheme_rtcall_Sl_s(const char *who, int src_type, prim_Sl_s f, Scheme_Object** g65, intptr_t g66)
+ Scheme_Object* scheme_rtcall_Sl_s(const char *who, int src_type, prim_Sl_s f, Scheme_Object** g73, intptr_t g74)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -212,8 +238,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_S0 = g65;
-    future->arg_l1 = g66;
+      future->arg_S0 = g73;
+    future->arg_l1 = g74;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -222,7 +248,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_l_s(const char *who, int src_type, prim_l_s f, intptr_t g67)
+ Scheme_Object* scheme_rtcall_l_s(const char *who, int src_type, prim_l_s f, intptr_t g75)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -237,7 +263,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_l0 = g67;
+      future->arg_l0 = g75;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -246,7 +272,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_bsi_v(const char *who, int src_type, prim_bsi_v f, Scheme_Bucket* g68, Scheme_Object* g69, int g70)
+ void scheme_rtcall_bsi_v(const char *who, int src_type, prim_bsi_v f, Scheme_Bucket* g76, Scheme_Object* g77, int g78)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -261,9 +287,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_b0 = g68;
-    future->arg_s1 = g69;
-    future->arg_i2 = g70;
+      future->arg_b0 = g76;
+    future->arg_s1 = g77;
+    future->arg_i2 = g78;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -272,7 +298,7 @@
   
   
 }
- void scheme_rtcall_iiS_v(const char *who, int src_type, prim_iiS_v f, int g71, int g72, Scheme_Object** g73)
+ void scheme_rtcall_iiS_v(const char *who, int src_type, prim_iiS_v f, int g79, int g80, Scheme_Object** g81)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -287,9 +313,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g71;
-    future->arg_i1 = g72;
-    future->arg_S2 = g73;
+      future->arg_i0 = g79;
+    future->arg_i1 = g80;
+    future->arg_S2 = g81;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -298,7 +324,7 @@
   
   
 }
- void scheme_rtcall_ss_v(const char *who, int src_type, prim_ss_v f, Scheme_Object* g74, Scheme_Object* g75)
+ void scheme_rtcall_ss_v(const char *who, int src_type, prim_ss_v f, Scheme_Object* g82, Scheme_Object* g83)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -313,8 +339,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g74;
-    future->arg_s1 = g75;
+      future->arg_s0 = g82;
+    future->arg_s1 = g83;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -323,7 +349,7 @@
   
   
 }
- void scheme_rtcall_b_v(const char *who, int src_type, prim_b_v f, Scheme_Bucket* g76)
+ void scheme_rtcall_b_v(const char *who, int src_type, prim_b_v f, Scheme_Bucket* g84)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -338,7 +364,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_b0 = g76;
+      future->arg_b0 = g84;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -347,7 +373,7 @@
   
   
 }
- Scheme_Object* scheme_rtcall_sl_s(const char *who, int src_type, prim_sl_s f, Scheme_Object* g77, intptr_t g78)
+ Scheme_Object* scheme_rtcall_sl_s(const char *who, int src_type, prim_sl_s f, Scheme_Object* g85, intptr_t g86)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -362,8 +388,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g77;
-    future->arg_l1 = g78;
+      future->arg_s0 = g85;
+    future->arg_l1 = g86;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -372,7 +398,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_iS_s(const char *who, int src_type, prim_iS_s f, int g79, Scheme_Object** g80)
+ Scheme_Object* scheme_rtcall_iS_s(const char *who, int src_type, prim_iS_s f, int g87, Scheme_Object** g88)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -387,8 +413,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g79;
-    future->arg_S1 = g80;
+      future->arg_i0 = g87;
+    future->arg_S1 = g88;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -397,7 +423,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_S_s(const char *who, int src_type, prim_S_s f, Scheme_Object** g81)
+ Scheme_Object* scheme_rtcall_S_s(const char *who, int src_type, prim_S_s f, Scheme_Object** g89)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -412,7 +438,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_S0 = g81;
+      future->arg_S0 = g89;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -421,7 +447,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_s_v(const char *who, int src_type, prim_s_v f, Scheme_Object* g82)
+ void scheme_rtcall_s_v(const char *who, int src_type, prim_s_v f, Scheme_Object* g90)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -436,8 +462,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g82;
-  send_special_result(future, g82);
+      future->arg_s0 = g90;
+  send_special_result(future, g90);
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
   
@@ -445,7 +471,7 @@
   
   
 }
- Scheme_Object* scheme_rtcall_iSi_s(const char *who, int src_type, prim_iSi_s f, int g83, Scheme_Object** g84, int g85)
+ Scheme_Object* scheme_rtcall_iSi_s(const char *who, int src_type, prim_iSi_s f, int g91, Scheme_Object** g92, int g93)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -460,9 +486,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g83;
-    future->arg_S1 = g84;
-    future->arg_i2 = g85;
+      future->arg_i0 = g91;
+    future->arg_S1 = g92;
+    future->arg_i2 = g93;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -471,7 +497,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_siS_v(const char *who, int src_type, prim_siS_v f, Scheme_Object* g86, int g87, Scheme_Object** g88)
+ void scheme_rtcall_siS_v(const char *who, int src_type, prim_siS_v f, Scheme_Object* g94, int g95, Scheme_Object** g96)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -486,9 +512,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g86;
-    future->arg_i1 = g87;
-    future->arg_S2 = g88;
+      future->arg_s0 = g94;
+    future->arg_i1 = g95;
+    future->arg_S2 = g96;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -497,7 +523,7 @@
   
   
 }
- void* scheme_rtcall_z_p(const char *who, int src_type, prim_z_p f, size_t g89)
+ void* scheme_rtcall_z_p(const char *who, int src_type, prim_z_p f, size_t g97)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -512,7 +538,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_z0 = g89;
+      future->arg_z0 = g97;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -521,7 +547,7 @@
   
   return retval;
 }
- Scheme_Object* scheme_rtcall_si_s(const char *who, int src_type, prim_si_s f, Scheme_Object* g90, int g91)
+ Scheme_Object* scheme_rtcall_si_s(const char *who, int src_type, prim_si_s f, Scheme_Object* g98, int g99)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -536,8 +562,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g90;
-    future->arg_i1 = g91;
+      future->arg_s0 = g98;
+    future->arg_i1 = g99;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -546,7 +572,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_sis_v(const char *who, int src_type, prim_sis_v f, Scheme_Object* g92, int g93, Scheme_Object* g94)
+ void scheme_rtcall_sis_v(const char *who, int src_type, prim_sis_v f, Scheme_Object* g100, int g101, Scheme_Object* g102)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -561,9 +587,60 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g92;
-    future->arg_i1 = g93;
-    future->arg_s2 = g94;
+      future->arg_s0 = g100;
+    future->arg_i1 = g101;
+    future->arg_s2 = g102;
+  
+  future_do_runtimecall(fts, (void*)f, 0, 1);
+  future = fts->thread->current_ft;
+  
+  
+  
+  
+}
+ int scheme_rtcall_ss_i(const char *who, int src_type, prim_ss_i f, Scheme_Object* g103, Scheme_Object* g104)
+   XFORM_SKIP_PROC
+{
+  Scheme_Future_Thread_State *fts = scheme_future_thread_state;
+  future_t *future;
+  double tm;
+  int retval;
+
+  future = fts->thread->current_ft;
+  future->prim_protocol = SIG_ss_i;
+  future->prim_func = f;
+  tm = get_future_timestamp();
+  future->time_of_request = tm;
+  future->source_of_request = who;
+  future->source_type = src_type;
+      future->arg_s0 = g103;
+    future->arg_s1 = g104;
+  
+  future_do_runtimecall(fts, (void*)f, 0, 1);
+  future = fts->thread->current_ft;
+  retval = future->retval_i;
+  future->retval_i = 0;
+  
+  return retval;
+}
+ void scheme_rtcall_iSp_v(const char *who, int src_type, prim_iSp_v f, int g105, Scheme_Object** g106, void* g107)
+   XFORM_SKIP_PROC
+{
+  Scheme_Future_Thread_State *fts = scheme_future_thread_state;
+  future_t *future;
+  double tm;
+  
+
+  future = fts->thread->current_ft;
+  future->prim_protocol = SIG_iSp_v;
+  future->prim_func = f;
+  tm = get_future_timestamp();
+  future->time_of_request = tm;
+  future->source_of_request = who;
+  future->source_type = src_type;
+      future->arg_i0 = g105;
+    future->arg_S1 = g106;
+    future->arg_p2 = g107;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
