@@ -50,8 +50,13 @@ static Scheme_Object *impersonate_vector(int argc, Scheme_Object **argv);
 static Scheme_Object *unsafe_vector_len (int argc, Scheme_Object *argv[]);
 static Scheme_Object *unsafe_vector_ref (int argc, Scheme_Object *argv[]);
 static Scheme_Object *unsafe_vector_set (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_vector_star_len (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_vector_star_ref (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_vector_star_set (int argc, Scheme_Object *argv[]);
 static Scheme_Object *unsafe_struct_ref (int argc, Scheme_Object *argv[]);
 static Scheme_Object *unsafe_struct_set (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_struct_star_ref (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_struct_star_set (int argc, Scheme_Object *argv[]);
 static Scheme_Object *unsafe_string_len (int argc, Scheme_Object *argv[]);
 static Scheme_Object *unsafe_string_ref (int argc, Scheme_Object *argv[]);
 static Scheme_Object *unsafe_string_set (int argc, Scheme_Object *argv[]);
@@ -164,7 +169,7 @@ scheme_init_unsafe_vector (Scheme_Env *env)
                                 | SCHEME_PRIM_IS_UNSAFE_FUNCTIONAL);
   scheme_add_global_constant("unsafe-vector-length", p, env);
 
-  p = scheme_make_immed_prim(unsafe_vector_len, "unsafe-vector*-length", 1, 1);
+  p = scheme_make_immed_prim(unsafe_vector_star_len, "unsafe-vector*-length", 1, 1);
   SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_UNARY_INLINED
                                 | SCHEME_PRIM_IS_UNSAFE_FUNCTIONAL);
   scheme_add_global_constant("unsafe-vector*-length", p, env);
@@ -174,7 +179,7 @@ scheme_init_unsafe_vector (Scheme_Env *env)
                                 | SCHEME_PRIM_IS_UNSAFE_FUNCTIONAL);
   scheme_add_global_constant("unsafe-vector-ref", p, env);
 
-  p = scheme_make_immed_prim(unsafe_vector_ref, "unsafe-vector*-ref", 2, 2);
+  p = scheme_make_immed_prim(unsafe_vector_star_ref, "unsafe-vector*-ref", 2, 2);
   SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_BINARY_INLINED
                                 | SCHEME_PRIM_IS_UNSAFE_FUNCTIONAL);
   scheme_add_global_constant("unsafe-vector*-ref", p, env);
@@ -183,7 +188,7 @@ scheme_init_unsafe_vector (Scheme_Env *env)
   SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_NARY_INLINED;
   scheme_add_global_constant("unsafe-vector-set!", p, env);  
 
-  p = scheme_make_immed_prim(unsafe_vector_set, "unsafe-vector*-set!", 3, 3);
+  p = scheme_make_immed_prim(unsafe_vector_star_set, "unsafe-vector*-set!", 3, 3);
   SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_NARY_INLINED;
   scheme_add_global_constant("unsafe-vector*-set!", p, env);  
 
@@ -192,7 +197,7 @@ scheme_init_unsafe_vector (Scheme_Env *env)
                                 | SCHEME_PRIM_IS_UNSAFE_FUNCTIONAL);
   scheme_add_global_constant("unsafe-struct-ref", p, env);
 
-  p = scheme_make_immed_prim(unsafe_struct_ref, "unsafe-struct*-ref", 2, 2);
+  p = scheme_make_immed_prim(unsafe_struct_star_ref, "unsafe-struct*-ref", 2, 2);
   SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_BINARY_INLINED
                                 | SCHEME_PRIM_IS_UNSAFE_FUNCTIONAL);
   scheme_add_global_constant("unsafe-struct*-ref", p, env);
@@ -201,7 +206,7 @@ scheme_init_unsafe_vector (Scheme_Env *env)
   SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_NARY_INLINED;
   scheme_add_global_constant("unsafe-struct-set!", p, env);  
 
-  p = scheme_make_immed_prim(unsafe_struct_set, "unsafe-struct*-set!", 3, 3);
+  p = scheme_make_immed_prim(unsafe_struct_star_set, "unsafe-struct*-set!", 3, 3);
   SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_NARY_INLINED;
   scheme_add_global_constant("unsafe-struct*-set!", p, env);  
 
@@ -235,6 +240,7 @@ scheme_init_unsafe_vector (Scheme_Env *env)
 }
 
 #define VECTOR_BYTES(size) (sizeof(Scheme_Vector) + ((size) - 1) * sizeof(Scheme_Object *))
+#define REV_VECTOR_BYTES(size) (((size) - (sizeof(Scheme_Vector) - sizeof(Scheme_Object *))) / sizeof(Scheme_Object *))
 
 Scheme_Object *
 scheme_make_vector (intptr_t size, Scheme_Object *fill)
@@ -250,7 +256,13 @@ scheme_make_vector (intptr_t size, Scheme_Object *fill)
   if (size < 1024) {
     vec = (Scheme_Object *)scheme_malloc_tagged(VECTOR_BYTES(size));
   } else {
-    vec = (Scheme_Object *)scheme_malloc_fail_ok(scheme_malloc_tagged, VECTOR_BYTES(size));
+    size_t sz;
+    sz = VECTOR_BYTES(size);
+    if (REV_VECTOR_BYTES(sz) != size)
+      /* overflow */
+      scheme_raise_out_of_memory(NULL, NULL);
+    else
+      vec = (Scheme_Object *)scheme_malloc_fail_ok(scheme_malloc_tagged, sz);
   }
 
   vec->type = scheme_vector_type;
@@ -283,7 +295,7 @@ make_vector (int argc, Scheme_Object *argv[])
 
   if ((len == -1) 
       /* also watch for overflow: */
-      || ((intptr_t)VECTOR_BYTES(len) < len)) {
+      || (REV_VECTOR_BYTES(VECTOR_BYTES(len)) != len)) {
     scheme_raise_out_of_memory("make-vector", "making vector of length %s",
 			       scheme_make_provided_string(argv[0], 1, NULL));
   }
@@ -875,12 +887,48 @@ static Scheme_Object *unsafe_vector_set (int argc, Scheme_Object *argv[])
   return scheme_void;
 }
 
+static Scheme_Object *unsafe_vector_star_len (int argc, Scheme_Object *argv[])
+{
+  Scheme_Object *vec = argv[0];
+  intptr_t n;
+  n = SCHEME_VEC_SIZE(vec);
+  return scheme_make_integer(n);
+}
+
+static Scheme_Object *unsafe_vector_star_ref (int argc, Scheme_Object *argv[])
+{
+  return SCHEME_VEC_ELS(argv[0])[SCHEME_INT_VAL(argv[1])];
+}
+
+static Scheme_Object *unsafe_vector_star_set (int argc, Scheme_Object *argv[])
+{
+  SCHEME_VEC_ELS(argv[0])[SCHEME_INT_VAL(argv[1])] = argv[2];
+  return scheme_void;
+}
+
 static Scheme_Object *unsafe_struct_ref (int argc, Scheme_Object *argv[])
+{
+  if (SCHEME_CHAPERONEP(argv[0]))
+    return scheme_struct_ref(argv[0], SCHEME_INT_VAL(argv[1]));
+  else
+    return ((Scheme_Structure *)argv[0])->slots[SCHEME_INT_VAL(argv[1])];
+}
+
+static Scheme_Object *unsafe_struct_set (int argc, Scheme_Object *argv[])
+{
+  if (SCHEME_CHAPERONEP(argv[0]))
+    scheme_struct_set(argv[0], SCHEME_INT_VAL(argv[1]), argv[2]);
+  else
+    ((Scheme_Structure *)argv[0])->slots[SCHEME_INT_VAL(argv[1])] = argv[2];
+  return scheme_void;
+}
+
+static Scheme_Object *unsafe_struct_star_ref (int argc, Scheme_Object *argv[])
 {
   return ((Scheme_Structure *)argv[0])->slots[SCHEME_INT_VAL(argv[1])];
 }
 
-static Scheme_Object *unsafe_struct_set (int argc, Scheme_Object *argv[])
+static Scheme_Object *unsafe_struct_star_set (int argc, Scheme_Object *argv[])
 {
   ((Scheme_Structure *)argv[0])->slots[SCHEME_INT_VAL(argv[1])] = argv[2];
   return scheme_void;
